@@ -39,13 +39,20 @@ import os, sys, csv, time
 import numpy
 import argparse
 from msproteomicstoolslib.format.SWATHScoringReader import *
-from msproteomicstoolslib.algorithms.alignment.AlignmentHelper import write_out_matrix_file
 import msproteomicstoolslib.format.TransformationCollection as transformations
+from msproteomicstoolslib.algorithms.alignment.Multipeptide import Multipeptide
+from msproteomicstoolslib.algorithms.alignment.SplineAligner import SplineAligner
+from msproteomicstoolslib.algorithms.alignment.AlignmentMST import getDistanceMatrix
+from msproteomicstoolslib.algorithms.PADS.MinimumSpanningTree import MinimumSpanningTree
+import msproteomicstoolslib.algorithms.alignment.AlignmentHelper as helper
+from msproteomicstoolslib.algorithms.alignment.BorderIntegration import \
+        integrationBorderShortestPath, integrationBorderShortestDistance, integrationBorderReference
 import msproteomicstoolslib.math.Smoothing as smoothing
 from feature_alignment import Experiment
 
 # The Window overlap which needs to be taken into account when calculating from which swath window to extract!
 SWATH_EDGE_SHIFT = 1
+VERBOSE = False
 
 class ImputeValuesHelper(object):
     """
@@ -57,7 +64,7 @@ class ImputeValuesHelper(object):
         """Select the correct chromatogram
 
         Args:
-            swath_chromatograms(dict): containing the objects pointing to the original chrom mzML (see run_impute_values)
+            swath_chromatograms(dict): containing the objects pointing to the original chrom mzML (see runImputeValues)
             mz(float): the mz value of the precursor
         """
         mz = mz + SWATH_EDGE_SHIFT
@@ -74,21 +81,6 @@ class ImputeValuesHelper(object):
                 res[k] = selected[0]
         return res
 
-    @staticmethod
-    def convert_to_this(orig_runid, target_runid, ref_id, rt, transformation_collection_):
-        """ Convert a retention time into one of the target RT space.
-        
-        Using the transformation collection
-        """
-        try:
-            normalized_space_rt = transformation_collection_.getTransformation(orig_runid, ref_id).predict( [rt] )[0]
-            return transformation_collection_.getTransformation(ref_id, target_runid).predict( [normalized_space_rt] )[0]
-        except AttributeError as e:
-            print "Could not convert from run %s to run %s (throug reference run %s)- are you sure you gave the correspoding trafo file with the --in parameter?" % (orig_runid, target_runid, ref_id)
-            print e
-            raise e
-
-
 class SwathChromatogramRun(object):
     """ A single SWATH LC-MS/MS run.
 
@@ -121,6 +113,14 @@ class SwathChromatogramCollection(object):
 
     Each single run is represented as a SwathChromatogramRun and accessible
     through a run id.
+
+    >>> mzml_files = ["file1.mzML", "file2.mzML"]
+    >>> runMapping = {"file1.mzML": "run1", "file2.mzML" : "run2"}
+    >>> chromatograms = SwathChromatogramCollection()
+    >>> chromatograms.parseFromMzML(mzml_files, runMapping)
+
+    >>> chromatogram = chromatograms.getChromatogram("run1", "ChromatogramId")
+    >>> chromatogram = chromatograms.getChromatogram("run2", "ChromatogramId")
     """
 
     def __init__(self):
@@ -150,8 +150,11 @@ class SwathChromatogramCollection(object):
             return None
         return self.allruns[runid].getChromatogram(chromid)
 
-    def parse(self, trafo_fnames):
-        """ Parse a set of different experiments.
+    def parseFromTrafoFiles(self, trafo_fnames):
+        """ Parse a set of different experiments from the .tr files
+
+        The mzML files belonging to the same run are assumed to be in the same
+        folder as the .tr files.
         """
         swath_chromatograms = {}
         for filename in trafo_fnames:
@@ -171,10 +174,117 @@ class SwathChromatogramCollection(object):
             self.allruns[runid] = swathrun
             print "Parsing chromatograms in", filename, "took %0.4fs" % (time.time() - start)
 
-    def get_runids(self):
+    def parseFromMzML(self, mzML_files, runIdMapping):
+        """ Parse a set of different experiments.
+
+        Args:
+            mzML_files(list(filename)): a list of filenames of the mzML files
+            runIdMapping(dict): a dictionary mapping each filename to a run id
+        """
+        swath_chromatograms = {}
+        for filename in mzML_files:
+            start = time.time()
+            runid = runIdMapping[filename]
+            swathrun = SwathChromatogramRun()
+            swathrun.parse(runid, [filename])
+            self.allruns[runid] = swathrun
+            print "Parsing chromatograms in", filename, "took %0.4fs" % (time.time() - start)
+
+    def getRunIDs(self):
         return self.allruns.keys()
 
-def run_impute_values(options, peakgroups_file, trafo_fnames):
+# Main entry points:
+# runSingleFileImputation
+# runImputeValues
+
+def runSingleFileImputation(options, peakgroups_file, mzML_file, method):
+    """Impute values across chromatograms
+
+    Args:
+        peakgroups_file(filename): CSV file containing all peakgroups
+        mzML_file(filename): mzML file containing chromatograms
+    Returns:
+        A tuple of:
+            new_exp(AlignmentExperiment): experiment containing the aligned peakgroups
+            multipeptides(list(AlignmentHelper.Multipeptide)): list of multipeptides
+
+    This function will read the csv file with all peakgroups as well as the
+    provided chromatogram file (.chrom.mzML). It will then try to impute
+    missing values for those peakgroups where no values is currently present,
+    reading the raw chromatograms.
+    """
+
+    # We do not want to exclude any peakgroups for noiseIntegration (we assume
+    # that alignment has already happened)
+    fdr_cutoff_all_pg = 1.0
+
+    start = time.time()
+    reader = SWATHScoringReader.newReader([peakgroups_file], options.file_format, readmethod="complete")
+    new_exp = Experiment()
+    new_exp.runs = reader.parse_files()
+    multipeptides = new_exp.get_all_multipeptides(fdr_cutoff_all_pg, verbose=False)
+    print("Parsing the peakgroups file took %ss" % (time.time() - start) )
+
+    mapping = {}
+    precursors_mapping = {}
+    sequences_mapping = {}
+    inferMapping([ mzML_file ], [ peakgroups_file ], mapping, precursors_mapping, sequences_mapping, verbose=False)
+    mapping_inv = dict([(v[0],k) for k,v in mapping.iteritems()])
+    if VERBOSE:
+        print mapping
+
+    # Do only a single run : read only one single file
+    start = time.time()
+    swath_chromatograms = SwathChromatogramCollection()
+    swath_chromatograms.parseFromMzML([ mzML_file ], mapping_inv)
+    print("Reading the chromatogram files took %ss" % (time.time() - start) )
+    assert len(swath_chromatograms.getRunIDs() ) == 1
+    rid = swath_chromatograms.getRunIDs()[0]
+
+    start = time.time()
+    initial_alignment_cutoff = 0.0001
+    max_rt_diff = 30
+    sd_data = -1 # We do not use the standard deviation data in this algorithm
+    tr_data = transformations.LightTransformationData()
+    spl_aligner = SplineAligner(initial_alignment_cutoff)
+
+    if method == "singleClosestRun":
+        tree_mapped = None
+
+        run_1 = [r for r in new_exp.runs if r.get_id() == rid][0]
+        dist_matrix = getDistanceMatrix(new_exp, multipeptides, spl_aligner, singleRowId=run_1.get_id())
+        print("Distance matrix took %ss" % (time.time() - start) )
+
+        start = time.time()
+        for run_0 in new_exp.runs:
+            helper.addDataToTrafo(tr_data, run_0, run_1, spl_aligner, multipeptides,
+                options.realign_method, max_rt_diff, sd_max_data_length=sd_data)
+
+    elif method == "singleShortestPath":
+        dist_matrix = None
+
+        tree = MinimumSpanningTree(getDistanceMatrix(new_exp, multipeptides, spl_aligner))
+        tree_mapped = [(new_exp.runs[a].get_id(), new_exp.runs[b].get_id()) for a,b in tree]
+        print("Distance matrix took %ss" % (time.time() - start) )
+
+        start = time.time()
+        for edge in tree:
+            helper.addDataToTrafo(tr_data, new_exp.runs[edge[0]], 
+                new_exp.runs[edge[1]], spl_aligner, multipeptides, 
+                options.realign_method, max_rt_diff, sd_max_data_length=sd_data)
+
+    else:
+        raise Exception("Unknown method: " + method)
+
+    print("Alignment took %ss" % (time.time() - start) )
+    start = time.time()
+    multipeptides = analyze_multipeptides(new_exp, multipeptides, swath_chromatograms,
+        tr_data, options.border_option, rid, tree=tree_mapped, mat=dist_matrix)
+    print("Analyzing the runs took %ss" % (time.time() - start) )
+
+    return new_exp, multipeptides
+
+def runImputeValues(options, peakgroups_file, trafo_fnames):
     """Impute values across chromatograms
 
     Args:
@@ -191,25 +301,12 @@ def run_impute_values(options, peakgroups_file, trafo_fnames):
     need to be in the same folder. It will then try to impute missing values
     for those peakgroups where no values is currently present, reading the raw
     chromatograms.
-
-    It produces a dict swath_chromatograms which can be described as following:
-
-        a dict whose key is the run_id (0_1, 0_2 etc) pointing to an all_swathes dict
-            the all_swathes has a key which is the floor m/z of the *first* precursor found in that run and a value of type pymzml.run.Reader:
-        { "0_0" : 
-            {501 : pymzml.run.Reader,
-             530 : pymzml.run.Reader,
-                [...]
-            }
-          "0_1" : 
-            {501 : pymzml.run.Reader,
-             530 : pymzml.run.Reader,
-                [...]
-            }
-            }
     """
 
+    # We do not want to exclude any peakgroups for noiseIntegration (we assume
+    # that alignment has already happened)
     fdr_cutoff_all_pg = 1.0
+
     start = time.time()
     reader = SWATHScoringReader.newReader([peakgroups_file], options.file_format, readmethod="complete")
     new_exp = Experiment()
@@ -225,17 +322,17 @@ def run_impute_values(options, peakgroups_file, trafo_fnames):
     # Read the datapoints and perform the smoothing
     print("Reading the trafo file took %ss" % (time.time() - start) )
     start = time.time()
-    transformation_collection_.initialize_from_data(reverse=True, use_scikit=options.use_scikit, use_linear=options.use_linear)
+    transformation_collection_.initialize_from_data(reverse=True, smoother=options.realign_method)
     print("Initializing the trafo file took %ss" % (time.time() - start) )
 
     if options.do_single_run and not options.dry_run:
         # Do only a single run : read only one single file
         start = time.time()
         swath_chromatograms = SwathChromatogramCollection()
-        swath_chromatograms.parse([ options.do_single_run ])
+        swath_chromatograms.parseFromTrafoFiles([ options.do_single_run ])
         print("Reading the chromatogram files took %ss" % (time.time() - start) )
-        assert len(swath_chromatograms.get_runids() ) == 1
-        rid = swath_chromatograms.get_runids()[0]
+        assert len(swath_chromatograms.getRunIDs() ) == 1
+        rid = swath_chromatograms.getRunIDs()[0]
         # 
         start = time.time()
         multipeptides = analyze_multipeptides(new_exp, multipeptides, swath_chromatograms,
@@ -244,7 +341,7 @@ def run_impute_values(options, peakgroups_file, trafo_fnames):
         return new_exp, multipeptides
 
     swath_chromatograms = SwathChromatogramCollection()
-    swath_chromatograms.parse(trafo_fnames)
+    swath_chromatograms.parseFromTrafoFiles(trafo_fnames)
     print("Reading the chromatogram files took %ss" % (time.time() - start) )
 
     if options.dry_run:
@@ -266,15 +363,15 @@ def run_impute_values(options, peakgroups_file, trafo_fnames):
             transformation_collection_, options.border_option)
     print("Analyzing the runs took %ss" % (time.time() - start) )
     return new_exp, multipeptides
-    
+
 def analyze_multipeptides(new_exp, multipeptides, swath_chromatograms, 
-                          transformation_collection_, border_option, onlyExtractFromRun=None):
+                          transformation_collection_, border_option, onlyExtractFromRun=None, tree=None, mat=None):
     """Analyze the multipeptides and impute missing values
 
     Args:
         new_exp(AlignmentExperiment): experiment containing the aligned peakgroups
         multipeptides(list(AlignmentHelper.Multipeptide)): list of multipeptides
-        swath_chromatograms(dict): containing the objects pointing to the original chrom mzML (see run_impute_values)
+        swath_chromatograms(dict): containing the objects pointing to the original chrom mzML (see runImputeValues)
         transformation_collection_(.TransformationCollection): specifying how to transform between retention times of different runs
 
     Returns:
@@ -282,108 +379,91 @@ def analyze_multipeptides(new_exp, multipeptides, swath_chromatograms,
 
     This function will update the input multipeptides and add peakgroups, imputing missing values 
     """
+
     # Go through all aligned peptides
-    run_ids = [r.get_id() for r in new_exp.runs]
     class CounterClass: pass
     cnt = CounterClass()
     cnt.integration_bnd_warnings = 0
     cnt.imputations = 0
     cnt.imputation_succ = 0
     cnt.peakgroups = 0
+
     print "Will work on %s peptides" % (len(multipeptides))
     for i,m in enumerate(multipeptides):
-        if i % 500 == 0: print "Done with %s out of %s" % (i, len(multipeptides))
+        if i % 500 == 0: 
+            print "Done with %s out of %s" % (i, len(multipeptides))
+        if VERBOSE:
+            print "do", m.get_peptides()[0].get_id()
 
         # Ensure that each run has either exactly one peakgroup (or zero)
-        if any([len(p.peakgroups) > 1 for p in m.get_peptides()] ):
+        if all ([pg.get_cluster_id() == -1 for p in m.get_peptides() for pg in p.get_all_peakgroups()]) and \
+           any([len(p.peakgroups) > 1 for p in m.get_peptides()] ):
             raise Exception("Found more than one peakgroup for peptide %s - \
-                \n is this after alignment or did you forget to run feature_alignment.py? beforehand" % (
+                \n is this after alignment or did you forget to run feature_alignment.py beforehand?" % (
                 m.get_id() ))
 
-        selected_pg = [p.peakgroups[0] for p in m.get_peptides() if len(p.peakgroups)==1 ] 
-        for rid in run_ids:
-            cnt.peakgroups += 1
+        clusters = set( [pg.get_cluster_id() for p in m.get_peptides() for pg in p.get_all_peakgroups()])
+        for cl in clusters:
+            selected_pg = [pg for p in m.get_peptides() for pg in p.get_all_peakgroups() if pg.get_cluster_id() == cl]
+            analyze_multipeptide_cluster(m, cnt, new_exp, swath_chromatograms, 
+                                      transformation_collection_, border_option, selected_pg, cl,
+                                      onlyExtractFromRun, tree, mat)
 
-            # Check whether we have a peakgroup for this run id, if yes we mark
-            # this peakgroup as selected for output, otherwise we try to impute.
-            if m.has_peptide(rid):
-                m.get_peptide(rid).peakgroups[0].select_this_peakgroup()
-            else:
-                cnt.imputations += 1
-                imputed=True
-
-                if not onlyExtractFromRun is None:
-                    if onlyExtractFromRun != rid:
-                        continue
-
-                # Select current run, compute right/left integration border and then integrate
-                current_run = [r for r in new_exp.runs if r.get_id() == rid][0]
-                border_l, border_r = determine_integration_border(new_exp, selected_pg, 
-                    rid, transformation_collection_, border_option)
-                newpg = integrate_chromatogram(selected_pg[0], current_run, swath_chromatograms,
-                                             border_l, border_r, cnt)
-                if newpg != "NA": 
-                    cnt.imputation_succ += 1
-                    precursor = GeneralPrecursor(newpg.get_value("transition_group_id"), rid)
-                    # Select for output, add to precursor
-                    newpg.select_this_peakgroup()
-                    precursor.add_peakgroup(newpg)
-                    m.insert(rid, precursor)
+        if all ([pg.get_cluster_id() == -1 for p in m.get_peptides() for pg in p.get_all_peakgroups()]):
+            # no cluster info was read in -> select all
+            for p in m.get_peptides():
+                for pg in p.get_all_peakgroups():
+                    pg.select_this_peakgroup()
 
     print "Imputations:", cnt.imputations, "Successful:", cnt.imputation_succ, "Still missing", cnt.imputations - cnt.imputation_succ
     print "WARNING: %s times: Chromatogram does not cover full range"  % (cnt.integration_bnd_warnings)
     print "Peakgroups:", cnt.peakgroups
     return multipeptides 
 
-def determine_integration_border(new_exp, selected_pg, rid, transformation_collection_, border_option):
-    """Determine the optimal integration border by taking the mean of all other peakgroup boundaries.
+def analyze_multipeptide_cluster(current_mpep, cnt, new_exp, swath_chromatograms, 
+                          transformation_collection_, border_option, selected_pg, cluster_id,
+                          onlyExtractFromRun=None, tree=None, mat=None):
 
-    Args:
-        new_exp(AlignmentExperiment): experiment containing the aligned peakgroups
-        selected_pg(list(GeneralPeakGroup)): list of selected peakgroups (e.g. those passing the quality threshold)
-        rid(String): current run id
-        transformation_collection_(.TransformationCollection): specifying how to transform between retention times of different runs
+        for rid in [r.get_id() for r in new_exp.runs]:
+            cnt.peakgroups += 1
 
-    Returns:
-        A tuple of (left_integration_border, right_integration_border) in the retention time space of the _reference_ run
-    """
-    current_run = [r for r in new_exp.runs if r.get_id() == rid][0]
-    ref_id = transformation_collection_.getReferenceRunID()
+            # Check whether we have a peakgroup for this run id, if yes we mark
+            # this peakgroup as selected for output, otherwise we try to impute.
+            if not any([pg.peptide.run.get_id() == rid for pg in selected_pg]):
 
-    pg_lefts = []
-    pg_rights = []
-    for pg in selected_pg:
+                # Skip if we should not extract from this run
+                if not onlyExtractFromRun is None:
+                    if onlyExtractFromRun != rid:
+                        continue
 
-        rwidth = float(pg.get_value("rightWidth"))
-        lwidth = float(pg.get_value("leftWidth"))
-        this_run_rwidth = ImputeValuesHelper.convert_to_this(pg.peptide.run.get_id(),
-            current_run.get_id(), ref_id, rwidth, transformation_collection_)
-        this_run_lwidth = ImputeValuesHelper.convert_to_this(pg.peptide.run.get_id(),
-            current_run.get_id(), ref_id, lwidth, transformation_collection_)
+                cnt.imputations += 1
 
-        pg_lefts.append(this_run_lwidth)
-        pg_rights.append(this_run_rwidth)
+                # Select current run, compute right/left integration border and then integrate
+                current_run = [r for r in new_exp.runs if r.get_id() == rid][0]
+                rmap = dict([(r.get_id(),i) for i,r in enumerate(new_exp.runs) ])
 
-    if border_option == "mean":
-        integration_left = numpy.mean(pg_lefts)
-        integration_right = numpy.mean(pg_rights)
-    elif border_option == "median":
-        integration_left = numpy.median(pg_lefts)
-        integration_right = numpy.median(pg_rights)
-    elif border_option == "max_width":
-        integration_left = numpy.min(pg_lefts)
-        integration_right = numpy.max(pg_rights)
-    else:
-        raise Exception("Unknown border determination option %s" % border_option)
-
-    std_warning_level = 20
-    if numpy.std(pg_rights) > std_warning_level or numpy.std(pg_lefts) > std_warning_level: 
-        pass
-        # TODO : what if they are not consistent ?
-        # print "Std is too large", pg_rights, pg_lefts
-        # print "overall mean, std", numpy.mean(pg_lefts), numpy.std(pg_lefts), numpy.mean(pg_rights), numpy.std(pg_rights)
-
-    return integration_left, integration_right
+                if VERBOSE: 
+                    print "Will try to fill NA in run", current_run.get_id(), "for peptide", current_mpep.get_peptides()[0].get_id()
+                if tree is not None:
+                    border_l, border_r = integrationBorderShortestPath(selected_pg, 
+                        rid, transformation_collection_, tree)
+                elif mat is not None:
+                    border_l, border_r = integrationBorderShortestDistance(selected_pg, 
+                        rid, transformation_collection_, mat, rmap)
+                else:
+                    border_l, border_r = integrationBorderReference(new_exp, selected_pg, rid, transformation_collection_, border_option)
+                newpg = integrate_chromatogram(selected_pg[0], current_run, swath_chromatograms,
+                                             border_l, border_r, cnt)
+                if newpg != "NA": 
+                    if VERBOSE: 
+                        print "Managed to fill NA in run", current_run.get_id(), \
+                          "with value", newpg.get_value("Intensity"), "/ borders", border_l, border_r
+                    cnt.imputation_succ += 1
+                    precursor = GeneralPrecursor(newpg.get_value("transition_group_id"), current_run)
+                    # Select for output, add to precursor
+                    newpg.setClusterID(cluster_id)
+                    precursor.add_peakgroup(newpg)
+                    current_mpep.insert(rid, precursor)
 
 def integrate_chromatogram(template_pg, current_run, swath_chromatograms, 
                            left_start, right_end, cnt):
@@ -411,7 +491,7 @@ def integrate_chromatogram(template_pg, current_run, swath_chromatograms,
     newpg = GeneralPeakGroup(newrow, current_run, template_pg.peptide)
     for element in ["transition_group_id"]:
         newpg.set_value(element, template_pg.get_value(element))
-    for element in ["decoy", "Sequence", "FullPeptideName", "Charge", "ProteinName", "nr_peaks", "m.z", "m/z"]:
+    for element in ["decoy", "Sequence", "FullPeptideName", "Charge", "ProteinName", "nr_peaks", "m.z", "m/z", "align_clusterid"]:
         try:
             newpg.set_value(element, template_pg.get_value(element))
         except KeyError:
@@ -483,27 +563,25 @@ def write_out(new_exp, multipeptides, outfile, matrix_outfile, single_outfile):
         # selected_peakgroups = [p.peakgroups[0] for p in m.get_peptides()]
         # if (len(selected_peakgroups)*2.0 / len(new_exp.runs) < fraction_needed_selected) : continue
         for p in m.get_peptides():
-            selected_pg = p.peakgroups[0]
-            if selected_pg is None: 
-                continue
-            if single_outfile:
-                # Only write the newly imputed ones ... 
-                if float(selected_pg.get_value("m_score")) > 1.0:
+            for selected_pg in p.peakgroups:
+                if single_outfile:
+                    # Only write the newly imputed ones ... 
+                    if float(selected_pg.get_value("m_score")) > 1.0:
+                        row_to_write = selected_pg.row
+                        writer.writerow(row_to_write)
+                else:
                     row_to_write = selected_pg.row
                     writer.writerow(row_to_write)
-            else:
-                row_to_write = selected_pg.row
-                writer.writerow(row_to_write)
 
     if len(matrix_outfile) > 0:
-        write_out_matrix_file(matrix_outfile, new_exp.runs, multipeptides, 0.0, style=options.matrix_output_method)
+        helper.write_out_matrix_file(matrix_outfile, new_exp.runs, multipeptides, 0.0, style=options.matrix_output_method)
 
 def handle_args():
     usage = "" #usage: %prog --in \"files1 file2 file3 ...\" [options]" 
     usage += "\nThis program will impute missing values"
 
     parser = argparse.ArgumentParser(description = usage )
-    parser.add_argument('--in', dest="infiles", nargs = '+', required=True, help = 'A list of transformation files in the same folder as the .chrom.mzML files')
+    parser.add_argument('--in', dest="infiles", nargs = '+', required=False, help = 'A list of transformation files in the same folder as the .chrom.mzML files')
     parser.add_argument("--peakgroups_infile", dest="peakgroups_infile", required=True, help="Infile containing peakgroups (outfile from feature_alignment.py)")
     parser.add_argument("--out", dest="output", required=True, help="Output file with imputed values")
     parser.add_argument('--file_format', default='openswath', help="Which input file format is used (openswath or peakview)")
@@ -512,8 +590,8 @@ def handle_args():
     parser.add_argument('--border_option', default='median', metavar="median", help="How to determine integration border (possible values: max_width, mean, median). Max width will use the maximal possible width (most conservative since it will overestimate the background signal).")
     parser.add_argument('--dry_run', action='store_true', default=False, help="Perform a dry run only")
     parser.add_argument('--cache_in_memory', action='store_true', default=False, help="Cache data from a single run in memory")
-    parser.add_argument('--use_linear', action='store_true', default=False, help="Use linear run alignment")
-    parser.add_argument('--use_scikit', action='store_true', default=False, help="Use datasmooth from scikit instead of R to re-align runs (needs to be installed)")
+    parser.add_argument('--method', dest='method', default="allTrafo", help="Which method to use (singleShortestPath, singleClosestRun, other)")
+    parser.add_argument('--realign_runs', dest='realign_method', default="splineR", help="How to re-align runs in retention time ('diRT': use only deltaiRT from the input file, 'linear': perform a linear regression using best peakgroups, 'splineR': perform a spline fit using R, 'splineR_external': perform a spline fit using R (start an R process using the command line, 'splinePy' use Python native spline from scikits.datasmooth (slow!), 'lowess': use Robust locally weighted regression (lowess smoother)")
     parser.add_argument('--verbosity', default=0, type=int, help="Verbosity")
     parser.add_argument('--do_single_run', default='', metavar="", help="Only do a single run")
 
@@ -525,7 +603,10 @@ def handle_args():
 
 def main(options):
     import time
-    new_exp, multipeptides = run_impute_values(options, options.peakgroups_infile, options.infiles)
+    if options.method in ["singleShortestPath", "singleClosestRun"]:
+        new_exp, multipeptides = runSingleFileImputation(options, options.peakgroups_infile, options.do_single_run, options.method)
+    else:
+        new_exp, multipeptides = runImputeValues(options, options.peakgroups_infile, options.infiles)
     if options.dry_run: return
     write_out(new_exp, multipeptides, options.output, options.matrix_outfile, options.do_single_run)
 
