@@ -113,28 +113,37 @@ def writeStandardConfigFile(filename):
 
     config.write()
 
+class Label(object):
+
+    def __init__(self, AA, deltamass, name):
+        self.AA = AA
+        self.deltamass = deltamass
+        self.name = name
 
 def readLabelingFile(labeling_file) :
     #Returns a dictionary of amino-acides (including also C-Term and N-Term) with the mass shifts due to an isotope labeling experiment.
     labeling = {}
 
-    file = open (labeling_file,"r")
-    while True :
-        line = file.readline()
-        if len(line) == 0 : break
+    cur_file = open (labeling_file,"r")
+    for line in cur_file:
+        # Skip empty lines or comments
+        if len(line.strip()) == 0 : continue
         if line[0] == '#' : continue
 
+        labelname = ''
         aminoacid = ''
         mass_shift = 0.0
 
-        sline = line.split('\t')
+        sline = line.strip().split('\t')
         if len(sline) >= 2 :
             aminoacid  = sline[0]
             mass_shift = float(sline[1])
+            if len(sline) >= 3:
+                labelname = sline[2]
 
-        labeling[aminoacid] = mass_shift
+        labeling[aminoacid] = Label(aminoacid, mass_shift, labelname)
 
-    file.close()
+    cur_file.close()
 
     print "Labeling file :" , labeling
     return labeling
@@ -643,8 +652,8 @@ def main(argv) :
     csv_headers_openswath = ['PrecursorMz', 'ProductMz', 'Tr_recalibrated', 'transition_name', 'CE',
                             'LibraryIntensity', 'transition_group_id', 'decoy', 'PeptideSequence', 'ProteinName', 
                             'Annotation', 'FullUniModPeptideName', 
-                            'PrecursorCharge', 'GroupLabel', 'UniprotID', 'FragmentType', 'FragmentCharge',
-                            'FragmentSeriesNumber']
+                            'PrecursorCharge', 'PeptideGroupLabel',   'UniprotID', 'FragmentType', 'FragmentCharge',
+                            'FragmentSeriesNumber', 'LabelType']
     
     csv_headers = csv_headers_peakview
 
@@ -989,11 +998,12 @@ def main(argv) :
                                     peak.intensity , spectrum.sequence , pep.getSequenceWithMods(code) , int(z_parent) ,
                                     peak.frg_serie , peak.frg_z , peak.frg_nr , iRT_experimental , protein_code1 , 'FALSE', protein_index[protein_desc] , 1 , protein_shared ]
                 if key == 'openswath' :
+                    transition_group_id = "%s_%s_%s" % (precursor_cnt, pep.getSequenceWithMods(code), int(z_parent))
                     transition = [precursorMZ, fragment_mz, iRT_experimental, "%s_%s_%s" % (transition_cnt, pep.getSequenceWithMods(code), int(z_parent)), '-1',
-                            peak.intensity, "%s_%s_%s" % (precursor_cnt, pep.getSequenceWithMods(code), int(z_parent)), 0, spectrum.sequence, protein_desc, 
+                            peak.intensity, transition_group_id, 0, spectrum.sequence, protein_desc, 
                             peak.peak_annotation, pep.getSequenceWithMods(code),
-                            int(z_parent), 'light', protein_code1, peak.frg_serie, peak.frg_z,
-                            peak.frg_nr ]
+                            int(z_parent), transition_group_id, protein_code1, peak.frg_serie, peak.frg_z,
+                            peak.frg_nr , 'light']
                 
                 filteredtransitions.append(transition)
             
@@ -1046,7 +1056,9 @@ def main(argv) :
                         frg_z                   = heavy_transition[10]
                         frg_number               = heavy_transition[11]
                     if key == 'openswath' :
-                        heavy_transition[4] = 'heavy'
+                        # heavy_transition[13] = 'heavy'
+                        heavy_transition[18] = 'heavy'
+                        heavy_transition[13] = heavy_transition[6]
                         precursorMZ_heavy      = heavy_transition[0]
                         fragment_mz_heavy      = heavy_transition[1]
                         sequence_heavy         = heavy_transition[8]
@@ -1054,13 +1066,33 @@ def main(argv) :
                         frg_serie              = heavy_transition[15]
                         frg_z                   = heavy_transition[16]
                         frg_number               = heavy_transition[17]
+
+                        # Modify full sequence with name of label
+                        fullseq = heavy_transition[11]
+                        fullseq_n = ""
+                        for aa in fullseq:
+                            fullseq_n += aa
+                            if aa in labeling and labeling[aa].name:
+                                fullseq_n += "(%s)" % labeling[aa].name
+
+                        # Store modified sequence and make transition id unique again
+                        heavy_transition[11] = fullseq_n
+                        heavy_transition[3] += 'heavy'
+                        heavy_transition[6] += 'heavy'
+
+
+
+
                                         
+                    # TODO also change full unimod peptide name     
+
+
                     #NOTE: This only works for y- and b- ions (AND their neutral losses). Other fragment series are ignored, and no heavy transition will be generated for them.
                     if frg_serie[0] not in ['y','b'] : continue
 
 
                     for aa in sequence_heavy :
-                        if aa in labeling : precursorMZ_heavy += labeling[aa] / z_parent
+                        if aa in labeling : precursorMZ_heavy += labeling[aa].deltamass / z_parent
 
                     frg_seq = sequence_heavy
                     #b series
@@ -1069,15 +1101,15 @@ def main(argv) :
                     if frg_serie == 'y': frg_seq = sequence_heavy[-frg_number:]
 
                     for aa in frg_seq :
-                        if aa in labeling : fragment_mz_heavy += labeling[aa] / frg_z
+                        if aa in labeling : fragment_mz_heavy += labeling[aa].deltamass / frg_z
 
                     #Check for C- and N-terminal labelings
                     if 'C-term' in labeling :
-                        precursorMZ_heavy += labeling['C-term'] / z_parent
-                        if frg_serie == 'y' : fragment_mz_heavy += labeling['C-term'] /frg_z
+                        precursorMZ_heavy += labeling['C-term'].deltamass / z_parent
+                        if frg_serie == 'y' : fragment_mz_heavy += labeling['C-term'].deltamass /frg_z
                     if 'N-term' in labeling :
-                        precursorMZ_heavy += labeling['N-term'] / z_parent
-                        if frg_serie == 'b' : fragment_mz_heavy += labeling['N-term'] /frg_z
+                        precursorMZ_heavy += labeling['N-term'].deltamass / z_parent
+                        if frg_serie == 'b' : fragment_mz_heavy += labeling['N-term'].deltamass /frg_z
 
 
                     #NOTE : if for any reason the Q3 mass has not changed (the labeling is not present in this fragment ion), it is not reported.
@@ -1139,6 +1171,8 @@ def do_filtering_and_write(filteredtransitions, writer, labeling, removeDuplicat
     if len(filteredtransitions) < mintransitions :
         filteredtransitions = [] #I don't think this is really necessary, just in case.
     if verbose : print "after removing num of transitions below the required minimum " , len(filteredtransitions)
+
+    adssfsfadsf
     
     #Write in the peakview input file (until the max number of transitions per peptide/z (the most intense N transitions)
     for index in range(0,min(len(filteredtransitions),maxtransitions)) :
@@ -1161,7 +1195,9 @@ def do_filtering_and_write(filteredtransitions, writer, labeling, removeDuplicat
                 frg_z                   = heavy_transition[10]
                 frg_number               = heavy_transition[11]
             if key == 'openswath' :
+                print heavy_transition
                 heavy_transition[4] = 'heavy'
+                print heavy_transition
                 precursorMZ_heavy      = heavy_transition[0]
                 fragment_mz_heavy      = heavy_transition[1]
                 sequence_heavy         = heavy_transition[8]
@@ -1175,7 +1211,7 @@ def do_filtering_and_write(filteredtransitions, writer, labeling, removeDuplicat
 
 
             for aa in sequence_heavy :
-                if aa in labeling : precursorMZ_heavy += labeling[aa] / z_parent
+                if aa in labeling : precursorMZ_heavy += labeling[aa].deltamass / z_parent
 
             frg_seq = sequence_heavy
             #b series
@@ -1184,15 +1220,15 @@ def do_filtering_and_write(filteredtransitions, writer, labeling, removeDuplicat
             if frg_serie == 'y': frg_seq = sequence_heavy[-frg_number:]
 
             for aa in frg_seq :
-                if aa in labeling : fragment_mz_heavy += labeling[aa] / frg_z
+                if aa in labeling : fragment_mz_heavy += labeling[aa].deltamass / frg_z
 
             #Check for C- and N-terminal labelings
             if 'C-term' in labeling :
-                precursorMZ_heavy += labeling['C-term'] / z_parent
-                if frg_serie == 'y' : fragment_mz_heavy += labeling['C-term'] /frg_z
+                precursorMZ_heavy += labeling['C-term'].deltamass / z_parent
+                if frg_serie == 'y' : fragment_mz_heavy += labeling['C-term'].deltamass /frg_z
             if 'N-term' in labeling :
-                precursorMZ_heavy += labeling['N-term'] / z_parent
-                if frg_serie == 'b' : fragment_mz_heavy += labeling['N-term'] /frg_z
+                precursorMZ_heavy += labeling['N-term'].deltamass / z_parent
+                if frg_serie == 'b' : fragment_mz_heavy += labeling['N-term'].deltamass /frg_z
 
 
             #NOTE : if for any reason the Q3 mass has not changed (the labeling is not present in this fragment ion), it is not reported.
