@@ -58,19 +58,22 @@ class Cluster:
       If there are multiple peakgroups selected, only the best one is retained.
       """
       run_ids = {}
-      if verbose: print "len pg ", len(self.peakgroups)
+      if verbose: 
+          print "Select one per run: len pg ", len(self.peakgroups)
+          # print "Best pg", bestpg.print_out()
       for pg in self.peakgroups:
           rid = pg.peptide.get_run_id()
           if rid in run_ids:
-              if verbose: print "have run id", rid, "multiple times", pg.get_fdr_score(),\
-                 "/", pg.get_normalized_retentiontime(), " vs ", \
-                 run_ids[rid].get_fdr_score(), "/", run_ids[rid].get_normalized_retentiontime()
+              if verbose: 
+                  print "have run id", rid, "multiple times", pg.get_fdr_score(),\
+                  "/", pg.get_normalized_retentiontime(), " vs ", \
+                  run_ids[rid].get_fdr_score(), "/", run_ids[rid].get_normalized_retentiontime()
               if run_ids[rid].get_fdr_score() > pg.get_fdr_score():
                   run_ids[rid] = pg
           else: run_ids[rid] = pg
       self.peakgroups = run_ids.values()
 
-    def get_total_score(self):
+    def getTotalScore(self):
       """
       Calculate the total score of a cluster (multiplication of probabilities)
       """
@@ -79,13 +82,19 @@ class Cluster:
         mult = mult * pg.get_fdr_score()
       return mult
 
-    def get_median_rt(self):
+    def getMedianRT(self):
       """
-      Calculate the total score of a cluster (multiplication of probabilities)
+      Calculate the median retention time of a cluster
       """
 
       return numpy.median( [pg.get_normalized_retentiontime() for pg in self.peakgroups] )
 
+    def getRTstd(self):
+      """
+      Calculate the standard deviation of the retention times
+      """
+      return numpy.std( [pg.get_normalized_retentiontime() for pg in self.peakgroups] )
+
 class AlignmentAlgorithm():
     """ A class of alignment algorithms
     """
@@ -109,35 +118,25 @@ class AlignmentAlgorithm():
             Alignment object with alignment statistics
         """
 
-        class AlignmentStatistics():
-            def __init__(self): 
-                self.nr_aligned = 0
-                self.nr_changed = 0
-                self.nr_quantified = 0
-                self.could_not_align = 0
-                self.fdr_good_in_all = 0
-        astas = AlignmentStatistics()
-
         for mpep in multipeptides:
 
             if mpep.all_above_cutoff(fdr_cutoff) and method in ["best_cluster_score", "best_overall"]:
                 # In non-global algorithms, do not re-align if the fdr is above the threshold in all runs
                 for p in mpep.get_peptides():
                     p.get_best_peakgroup().select_this_peakgroup()
-                    astas.nr_quantified += 1
                 continue
 
             if method == "global_best_cluster_score" or method == "best_cluster_score":
-                self.align_features_cluster_(mpep, astas, rt_diff_cutoff, fdr_cutoff, aligned_fdr_cutoff, method)
+                self._align_features_cluster(mpep, rt_diff_cutoff, fdr_cutoff, aligned_fdr_cutoff, method)
             elif method == "global_best_overall" or method == "best_overall":
-                self.align_features_best_(mpep, astas, rt_diff_cutoff, fdr_cutoff, aligned_fdr_cutoff, method)
+                self._align_features_best(mpep, rt_diff_cutoff, fdr_cutoff, aligned_fdr_cutoff, method)
             else:
                 raise Exception("Method '%s' unknown" % method)
-        return astas
 
-    def align_features_cluster_(self, m, a, rt_diff_cutoff, fdr_cutoff, aligned_fdr_cutoff, method):
-        verb=self.verbose
-        if verb: print "00000000000000000000000000000000000 new peptide ", m.get_peptides()[0].sequence
+    def _align_features_cluster(self, m, rt_diff_cutoff, fdr_cutoff, aligned_fdr_cutoff, method):
+        verb = self.verbose
+
+        if verb: print "00000000000000000000000000000000000 new peptide (cluster)", m.get_peptides()[0].get_id()
 
         # i) get all RTs above the cutoff
         for p in m.get_peptides(): # loop over runs
@@ -164,37 +163,34 @@ class AlignmentAlgorithm():
 
         if verb: print "==== Clusters "
         # make sure only one is selected from each run...
-        for c in clusters_rt_obj: 
+        for i,c in enumerate(clusters_rt_obj): 
             c.select_one_per_run(self.verbose)
             if verb:
-                print " - Cluster with score", c.get_total_score(), "at", c.get_median_rt()
-                for pg in c.peakgroups: print pg.get_normalized_retentiontime(), pg.peptide.run.get_id()
+                print " - Cluster with score", c.getTotalScore(), "at", \
+                  c.getMedianRT(), "+/-", c.getRTstd() , "(norm_score %s)" %\
+                  (float(c.getTotalScore())/((aligned_fdr_cutoff/2)**len(c.peakgroups)))
+                for pg in c.peakgroups: 
+                    print "   = Have member", pg.print_out()
           
-        if len(clusters_rt_obj) == 1 :
-            # great, only one cluster
-            bestcluster = clusters_rt_obj[0]
-        else:
-            # select the best cluster
-            bestcluster = clusters_rt_obj[0] # self.getBestCluster(clusters_rt, rt_maximal_distance, method = clusterMode)
-            for cluster in clusters_rt_obj:
-                if cluster.get_total_score() < bestcluster.get_total_score(): bestcluster = cluster
+        # Get best cluster by length-normalized best score.
+        #   Length normalization divides the score by the expected probability
+        #   values if all peakgroups were chosen randomly (assuming equal
+        #   probability between 0 and aligned_fdr_cutoff, the expected value
+        #   for a random peakgroup is "aligned_fdr_cutoff/2") and thus the
+        #   expected random value of n peakgroups would be (aligned_fdr_cutoff/2)^n
+        bestcluster = min(clusters_rt_obj, key=(lambda x: x.getTotalScore()/(((aligned_fdr_cutoff/2)**len(c.peakgroups)))) )
 
-        for pg in bestcluster.peakgroups:
-          a.nr_quantified += 1
-          pg.select_this_peakgroup()
-          if pg.get_fdr_score() > fdr_cutoff:
-              a.nr_aligned += 1
-              if pg.get_normalized_retentiontime() != pg.peptide.get_best_peakgroup().get_normalized_retentiontime():
-                  a.nr_changed += 1
-                  if verb: print "FDR new align", pg.peptide.get_best_peakgroup().print_out(), "\tnew ====> ", pg.print_out()
-              else:
-                  if verb: print "FDR boost", pg.peptide.get_best_peakgroup().print_out(), " old ====> ", pg.print_out()
-          else:
-            if verb: print "no need to align", pg.print_out()
+        clusters_rt_obj.sort(lambda x,y: 
+                             cmp(x.getTotalScore()/((aligned_fdr_cutoff/2)**len(x.peakgroups)),
+                             y.getTotalScore()/((aligned_fdr_cutoff/2)**len(y.peakgroups))) )
+        for i,c in enumerate(clusters_rt_obj): 
+            for pg in c.peakgroups:
+                pg.setClusterID(i+1)
 
-    def align_features_best_(self, m, a, rt_diff_cutoff, fdr_cutoff, aligned_fdr_cutoff, method):
-        verb=self.verbose
-        if verb: print "00000000000000000000000000000000000 new peptide ", m.get_peptides()[0].sequence
+    def _align_features_best(self, m, rt_diff_cutoff, fdr_cutoff, aligned_fdr_cutoff, method):
+        verb = self.verbose
+
+        if verb: print "00000000000000000000000000000000000 new peptide (best overall)", m.get_peptides()[0].get_id()
 
         # If we just choose the cluster with the "best" peptide, we find find the best peptide over all runs
         best = m.find_best_peptide_pg()
@@ -202,13 +198,14 @@ class AlignmentAlgorithm():
         if verb: print "=====\nFDR best", best.print_out()
 
         for p in m.get_peptides(): # loop over runs
+            current_best_pg = p.get_best_peakgroup()
 
-            pg = p.get_best_peakgroup()
-            if pg.get_fdr_score() < fdr_cutoff and method == "best_overall":
+            if current_best_pg.get_fdr_score() < fdr_cutoff and method == "best_overall":
                 # The pg is below the fdr cutoff, we just take it and go with it in "best overall"
-                pg.select_this_peakgroup()
-                a.nr_quantified += 1
-                if verb: print "FDR below", p.get_id(), pg.get_fdr_score(), pg.get_normalized_retentiontime() #, pg.get_value("RT"), pg.get_value("rt_score") # rt_score = delta iRT
+                current_best_pg.select_this_peakgroup()
+                if verb: 
+                    print " == Selected peakgroup ", current_best_pg.print_out()
+
                 continue
 
             # #################################################################
@@ -217,8 +214,8 @@ class AlignmentAlgorithm():
             #   - of those select the peakgroup with the best score
             #   - if the best-scoring peakgroup is acceptable (<aligned_fdr_cutoff), mark it as selected
 
-            matching_peakgroups = [pg for pg in p.get_all_peakgroups() 
-                if (abs(float(pg.get_normalized_retentiontime()) - float(best_rt_diff)) < rt_diff_cutoff)]
+            matching_peakgroups = [pg_ for pg_ in p.get_all_peakgroups() 
+                if (abs(float(pg_.get_normalized_retentiontime()) - float(best_rt_diff)) < rt_diff_cutoff)]
 
             if len(matching_peakgroups) == 0: 
                 continue
@@ -228,16 +225,13 @@ class AlignmentAlgorithm():
             if bestScoringPG.get_fdr_score() < aligned_fdr_cutoff: 
                 bestScoringPG.select_this_peakgroup()
 
-                a.nr_aligned += 1
-                a.nr_quantified += 1
-                if pg.get_normalized_retentiontime() != bestScoringPG.get_normalized_retentiontime():
-                  a.nr_changed += 1
-                  if verb: print "FDR new align", pg.print_out(), "\tnew ====> ", bestScoringPG.print_out()
+                if current_best_pg.get_normalized_retentiontime() != bestScoringPG.get_normalized_retentiontime():
+                  if verb: 
+                      print "FDR new align", current_best_pg.print_out(), "\tnew ====> ", bestScoringPG.print_out()
                 else:
-                  if verb: print "FDR boost", pg.print_out(), " old ====> ", bestScoringPG.print_out()
+                  if verb: print "FDR boost", current_best_pg.print_out(), " old ====> ", bestScoringPG.print_out()
             else:
-                if verb: print "could not align", pg.peptide.run.get_id(), pg.peptide.run.orig_filename, "best rt_diff was ", \
+                if verb: print "could not align", current_best_pg.peptide.run.get_id(), current_best_pg.peptide.run.orig_filename, "best rt_diff was ", \
                       abs(float(bestScoringPG.get_normalized_retentiontime()) - float(best_rt_diff)), "best score", \
                       bestScoringPG.get_fdr_score() 
-                a.could_not_align += 1
 
