@@ -58,14 +58,12 @@ class SplineAligner():
     >>> spl_aligner = SplineAligner()
     >>> transformations = spl_aligner.rt_align_all_runs(this_exp, multipeptides, options.alignment_score, options.use_scikit)
     """
-    def __init__(self, alignment_fdr_threshold = 0.0001, use_scikit=False, 
-                 use_linear=False, use_external_r=False, external_r_tmpdir=None):
+    def __init__(self, alignment_fdr_threshold = 0.0001, smoother="lowess", external_r_tmpdir=None, maxdata=-1):
       self.transformation_collection = TransformationCollection()
       self.alignment_fdr_threshold_ = alignment_fdr_threshold
-      self.use_scikit_ = use_scikit
-      self.use_linear_ = use_linear
-      self.use_external_r_ = use_external_r
+      self.smoother = smoother
       self.tmpdir_ = external_r_tmpdir
+      self.max_data_ = maxdata
 
     def _determine_best_run(self, experiment):
 
@@ -84,59 +82,84 @@ class SplineAligner():
         print "Found best run", bestrun, "with %s features above the cutoff of %s%%" % (maxcount, self.alignment_fdr_threshold_)
         return [r for r in experiment.runs if r.get_id() == bestrun][0]
 
-    def _spline_align_runs(self, bestrun, run, multipeptides):
-        """Will align run against bestrun"""
+    def _getRTData(self, bestrun, run, multipeptides):
+        """ Return retention time data for reference and slave run """
 
-        # get those peptides we want to use for alignment => for this use the mapping
         # data1 = reference data (master)
         # data2 = data to be aligned (slave)
         data1 = []
         data2 = []
+
+        data_tmp = []
         for m in multipeptides:
+
             try: 
+                len_ali = len([pg for pg in m.get_peptide(run.get_id()).get_all_peakgroups() 
+                               if pg.get_fdr_score() < self.alignment_fdr_threshold_])
+                len_ref = len([pg for pg in m.get_peptide(bestrun.get_id()).get_all_peakgroups() 
+                               if pg.get_fdr_score() < self.alignment_fdr_threshold_])
+
+                # Do not consider peakgroups that are missing in one run
+                # Do not consider peakgroups that have more than one good peakgroup
+                if len_ali != 1 or len_ref != 1:
+                    continue
+
                 ref_pep = m.get_peptide(bestrun.get_id()).get_best_peakgroup()
                 align_pep = m.get_peptide(run.get_id()).get_best_peakgroup()
             except KeyError: 
                 # it is possible that for some, no peak group exists in this run
                 continue
-            if ref_pep.peptide.get_decoy() or align_pep.peptide.get_decoy(): continue
-            if ref_pep.get_fdr_score() < self.alignment_fdr_threshold_ and align_pep.get_fdr_score() < self.alignment_fdr_threshold_:
-                data1.append(ref_pep.get_normalized_retentiontime())
-                data2.append(align_pep.get_normalized_retentiontime())
 
-        # from run to bestrun
-        self.transformation_collection.addTransformationData([data2, data1], run.get_id(), bestrun.get_id() )
+            # Do not use decoy peptides
+            if ref_pep.peptide.get_decoy() or align_pep.peptide.get_decoy(): 
+                continue
 
-        all_pg = []
-        for pep in run.all_peptides.values():
-            all_pg.extend( [ (pg.get_normalized_retentiontime(), pg.get_feature_id()) for pg in pep.get_all_peakgroups()] )
+            if ref_pep.get_fdr_score() < self.alignment_fdr_threshold_ and \
+               align_pep.get_fdr_score() < self.alignment_fdr_threshold_:
 
-        rt_eval = [ pg[0] for pg in all_pg]
+                # data1.append(ref_pep.get_normalized_retentiontime())
+                # data2.append(align_pep.get_normalized_retentiontime())
+                data_tmp.append( (
+                    ref_pep.get_fdr_score(), 
+                    ref_pep.get_normalized_retentiontime(), 
+                    align_pep.get_normalized_retentiontime() 
+                ) )
+
+        maxdata = self.max_data_
+        if maxdata == -1:
+            # -1 means take all data
+            maxdata = len(data_tmp)
+
+        for fdr, d1, d2 in sorted(data_tmp)[:maxdata]:
+            data1.append(d1)
+            data2.append(d2)
+
+        return data1,data2
+
+    def _spline_align_runs(self, bestrun, run, multipeptides):
+        """Will align run against bestrun"""
+
+        sm = smoothing.getSmoothingObj(smoother = self.smoother, tmpdir = self.tmpdir_)
+
+        # get those peptides we want to use for alignment => for this use the mapping
+        # data1 = reference data (master)
+        # data2 = data to be aligned (slave)
+        data1,data2 = self._getRTData(bestrun, run, multipeptides)
 
-        # Since we want to predict how to convert from slave to master, slave
-        # is first and master is second.
-        sm = smoothing.get_smooting_operator(use_scikit = self.use_scikit_, 
-                                             use_linear = self.use_linear_,
-                                             use_external_r = self.use_external_r_,
-                                             tmpdir = self.tmpdir_)
         if len(data2) < 2:
             print "No common identifications between %s and %s. Only found %s features below a cutoff of %s" % ( 
                 run.get_id(), bestrun.get_id(), len(data1), self.alignment_fdr_threshold_)
             print "If you ran the feature_alignment.py script, try to skip the re-alignment step (e.g. remove the --realign_runs option)." 
             raise Exception("Not enough datapoints (less than 2 datapoints).")
+
+        # Since we want to predict how to convert from slave to master, slave
+        # is first and master is second.
         sm.initialize(data2, data1)
-        aligned_result = sm.predict(rt_eval)
-
         data2_aligned = sm.predict(data2)
-        self.transformation_collection.addTransformedData(data2_aligned, run.get_id(), bestrun.get_id() )
 
-        # The two methods produce very, very similar results
-        # but R is faster => prefer to use R when possible.
-        # hist(aligned_result - aligned_result_2, 100)
-        # numpy.std(aligned_result - aligned_result_2)
-        # 0.66102016517870454
-        # numpy.median(aligned_result - aligned_result_2)
-        # -0.020456989235640322
+        # Store transformation in collection (from run to bestrun)
+        self.transformation_collection.addTransformationData([data2, data1], run.get_id(), bestrun.get_id() )
+        self.transformation_collection.addTransformedData(data2_aligned, run.get_id(), bestrun.get_id() )
 
         stdev = numpy.std(numpy.array(data1) - numpy.array(data2_aligned))
         median = numpy.median(numpy.array(data1) - numpy.array(data2_aligned))
@@ -148,8 +171,13 @@ class SplineAligner():
         d[bestrun.get_id()] = [stdev, median]
         self.transformation_error.transformations[ run.get_id() ] = d
 
-        # now re-populate the peptide data!
+        # Now predict on _all_ data and write this back to the data
         i = 0
+        all_pg = []
+        for pep in run.all_peptides.values():
+            all_pg.extend( [ (pg.get_normalized_retentiontime(), pg.get_feature_id()) for pg in pep.get_all_peakgroups()] )
+        rt_eval = [ pg[0] for pg in all_pg]
+        aligned_result = sm.predict(rt_eval)
         for pep in run.all_peptides.values():
             mutable = [list(pg) for pg in pep.peakgroups_]
             for k in range(len(mutable)):
