@@ -53,405 +53,9 @@ Doc :
     this_exp.runs = reader.parse_files(options.realign_runs)
 """
 
-#
-# The data structures of the reader
-#
-class PeakGroupBase(object):
-
-    def __init__(self):
-        self.fdr_score = None
-        self.normalized_retentiontime = None
-        self.id_ = None
-        self.intensity_ = None
-        self.selected_ = False
-  
-    def get_value(self, value):
-        raise Exception("Needs implementation")
-
-    def set_value(self, key, value):
-        raise Exception("Needs implementation")
-
-    def set_fdr_score(self, fdr_score):
-        self.fdr_score = fdr_score
-
-    def get_fdr_score(self):
-        return self.fdr_score
-
-    def set_normalized_retentiontime(self, normalized_retentiontime):
-        self.normalized_retentiontime = float(normalized_retentiontime)
-
-    def get_normalized_retentiontime(self):
-        return self.normalized_retentiontime
-
-    def set_feature_id(self, id_):
-        self.id_ = id_
-
-    def get_feature_id(self):
-        return self.id_
-
-    def set_intensity(self, intensity):
-      self.intensity_ = intensity
-
-    def get_intensity(self):
-        return self.intensity_
-
-    # Selected
-    def is_selected(self):
-        return self.selected_
-
-    def select_this_peakgroup(self):
-        self.selected_ = True
-
-    def unselect_this_peakgroup(self):
-        self.selected_ = False
-
-
-class MinimalPeakGroup(PeakGroupBase):
-    """
-    A single peakgroup that is defined by a retention time in a chromatogram
-    of multiple transitions. Additionally it has an fdr_score and it has an
-    aligned RT (e.g. retention time in normalized space).
-    A peakgroup can be selected for quantification or not.
-    
-    Note that for performance reasons, the peakgroups are created on-the-fly
-    and not stored as objects but rather as tuples in "Peptide".
-
-    Each peak group has a unique id, a score (fdr score usually), a retention
-    time as well as a back-reference to the precursor that generated the
-    peakgroup.
-    In this case, the peak group can also be selected or unselected.
-    
-    """
-    def __init__(self, unique_id, fdr_score, assay_rt, selected, peptide, intensity=None, dscore=None):
-      super(MinimalPeakGroup, self).__init__()
-      self.id_ = unique_id
-      self.fdr_score = fdr_score
-      self.normalized_retentiontime = assay_rt 
-      self.selected_ = selected
-      self.peptide = peptide
-      self.intensity_ = intensity
-      self.dscore_ = dscore
-  
-    ## Print
-    def print_out(self):
-        # return self.run.get_id() + "/" + self.get_id() + " " + str(self.get_fdr_score()) + " " + str(self.get_normalized_retentiontime()) + " " + str(self.get_value("RT")) + " " + str(self.get_value("rt_score")) # rt_score = delta iRT
-        return self.peptide.run.get_id() + "/" + self.get_feature_id() + " score:" + str(self.get_fdr_score()) + " RT:" + str(self.get_normalized_retentiontime()) # + " " + str(self.get_value("RT")) + " " + str(self.get_value("rt_score")) # rt_score = delta iRT
-
-    # Do not allow setting of any parameters (since data is not stored here)
-    def set_fdr_score(self, fdr_score):
-        raise Exception("Cannot set in immutable object")
-
-    def set_normalized_retentiontime(self, normalized_retentiontime):
-        raise Exception("Cannot set in immutable object")
-
-    def set_feature_id(self, id_):
-        raise Exception("Cannot set in immutable object")
-
-    def set_intensity(self, intensity):
-        raise Exception("Cannot set in immutable object")
-
-    def get_dscore(self):
-        return self.dscore_
-
-    ## Select / De-select peakgroup
-    def select_this_peakgroup(self):
-        self.selected_ = True
-        self.peptide.select_pg(self.get_feature_id())
-
-    def unselect_this_peakgroup(self):
-        self.selected_ = False
-        self.peptide.unselect_pg(self.get_feature_id() )
-
-class GuiPeakGroup(PeakGroupBase):
-    """
-    A single peakgroup that is defined by a retention time in a chromatogram
-    of multiple transitions.
-    """
-    def __init__(self, fdr_score, intensity, leftWidth, rightWidth, peptide):
-      super(PeakGroupBase, self).__init__()
-      self.fdr_score = fdr_score
-      self.intensity_ = intensity
-      self.leftWidth_ = leftWidth
-      self.rightWidth_ = rightWidth
-      self.peptide = peptide
-  
-    def get_value(self, value):
-        if value == "m_score":
-            return self.fdr_score
-        elif value == "Intensity":
-            return self.intensity_
-        elif value == "rightWidth":
-            return self.rightWidth_
-        elif value == "leftWidth":
-            return self.leftWidth_
-        elif value == "FullPeptideName":
-            return self.peptide.sequence
-        elif value == "Charge":
-            return self.peptide.charge
-        else:
-            raise Exception("Do not have value " + value)
-
-class GeneralPeakGroup(PeakGroupBase):
-
-    def __init__(self, row, run, peptide):
-      super(GeneralPeakGroup, self).__init__()
-      self.row = row
-      self.run = run
-      self.peptide = peptide
-
-    def get_value(self, value):
-        return self.row[self.run.header_dict[value]]
-
-    def set_value(self, key, value):
-        if value is None:
-            value = "NA"
-        self.row[self.run.header_dict[key]] = value
-
-    def get_dscore(self):
-        return self.get_value("d_score")
-  
-class PrecursorBase(object):
-    def __init__(self, this_id, run):
-        raise NotImplemented
-
-    def get_id(self):
-        return self.id 
-  
-    def get_decoy(self):
-        return self._decoy
-
-    def set_decoy(self, decoy):
-        if decoy == "FALSE" or decoy == "0":
-            self._decoy = False
-        elif decoy == "TRUE" or decoy == "1":
-            self._decoy = True
-        else:
-            raise Exception("Unknown decoy classifier '%s', please check your input data!" % decoy)
-  
-    # store information about the peakgroup - tuples (e.g. whether they are selected)
-    def select_pg(self, this_id):
-        raise NotImplemented
-
-    def unselect_pg(self, id):
-        raise NotImplemented
-
-    def get_best_peakgroup(self):
-        raise NotImplemented
-
-    def get_selected_peakgroup(self):
-        raise NotImplemented
-
-    def get_all_peakgroups(self):
-        raise NotImplemented
-  
-    def find_closest_in_iRT(self, delta_assay_rt):
-        raise NotImplemented
-
-class GeneralPrecursor(PrecursorBase):
-    # A collection of peakgroups that belong to the same precursor and belong
-    # to one run.
-    # A peptide can return its best transition group, the selected peakgroup,
-    # or can return the transition group that is closest to a given iRT time.
-    def __init__(self, this_id, run):
-        self.id = this_id
-        self.peakgroups = []
-        self.run = run
-        self._decoy = False
-  
-    def add_peakgroup(self, peakgroup):
-        self.peakgroups.append(peakgroup)
-  
-    def get_run_id(self):
-      return self.run.get_id()
-  
-    def append(self, transitiongroup):
-        assert self.id == transitiongroup.get_id()
-        self.peakgroups.append(transitiongroup)
-  
-    def get_best_peakgroup(self):
-        """ Return the best peakgroup according to fdr score
-        """
-        if len(self.peakgroups) == 0: return None
-        best_score = self.peakgroups[0].get_fdr_score()
-        result = self.peakgroups[0]
-        for peakgroup in self.peakgroups:
-            if peakgroup.get_fdr_score() <= best_score:
-                best_score = peakgroup.get_fdr_score()
-                result = peakgroup
-        return result
-  
-    def get_selected_peakgroup(self):
-        # return the selected peakgroup of this peptide, we can only select 1 or
-        # zero groups per chromatogram!
-        selected = [peakgroup for peakgroup in self.peakgroups if peakgroup.is_selected()]
-        assert len(selected) < 2
-        if len(selected) == 1:
-          return selected[0]
-        else: 
-            return None
-
-    def get_all_peakgroups(self):
-        return self.peakgroups
-  
-    def find_closest_in_iRT(self, delta_assay_rt):
-      return min(self.peakgroups, key=lambda x: abs(float(x.get_normalized_retentiontime()) - float(delta_assay_rt)))
-
-class Precursor(PrecursorBase):
-    """
-    A collection of peakgroups that belong to the same precursor and belong
-    to one run.
-    A peptide can return its best transition group, the selected peakgroup,
-    or can return the transition group that is closest to a given iRT time.
-    Its id is the transition_group_id (e.g. the id of the chromatogram)
-    
-    For memory reasons, we store all information about the peakgroup in a
-    tuple (invariable). This tuple contains a unique feature id, a score and
-    a retention time. Additionally, we also store, whether the feature was
-    selected or not.
-
-    A peakgroup has the following attributes: 
-        - an identifier that is unique among all other precursors 
-        - a set of peakgroups 
-        - a backreference to the run it belongs to
-    """
-    def __init__(self, this_id, run):
-        self.id = this_id  
-        self.peakgroups = []
-        self.run = run
-        self.peakgroups_ = []
-        self.selected_ = []
-        self._decoy = False
-  
-    def __str__(self):
-        return "%s (run %s)" % (self.id, self.run)
-
-    def add_peakgroup_tpl(self, pg_tuple, tpl_id):
-        """Adds a peakgroup to this precursor.
-
-        The peakgroup should be a tuple of length 4 with the following components:
-            0. id
-            1. quality score (FDR)
-            2. retention time (normalized)
-            3. intensity
-            (4. d_score optional)
-        """
-        assert self.id == tpl_id # Check that the peak group is added to the correct precursor
-        if len(pg_tuple) == 4:
-            pg_tuple = pg_tuple + (None,)
-        assert len(pg_tuple) == 5
-        self.peakgroups_.append(pg_tuple)
-        self.selected_.append(False)
-
-    def get_id(self):
-        return self.id 
-  
-    def get_run_id(self):
-      return self.run.get_id()
-    
-    def get_decoy(self):
-        return self._decoy
-
-    # store information about the peakgroup - tuples (e.g. whether they are selected)
-    def select_pg(self, this_id):
-        pg_id = [i for i,pg in enumerate(self.peakgroups_) if pg[0] == this_id]
-        assert len(pg_id) == 1
-        self.selected_[pg_id[0]] = True
-
-    def unselect_pg(self, this_id):
-        pg_id = [i for i,pg in enumerate(self.peakgroups_) if pg[0] == this_id]
-        assert len(pg_id) == 1
-        self.selected_[pg_id[0]] = False
-
-    def unselect_all(self):
-        for i in range(len(self.selected_)) : self.selected_[i] = False
-
-    def get_best_peakgroup(self):
-        if len(self.peakgroups_) == 0: return None
-        best_score = self.peakgroups_[0][1]
-        result = self.peakgroups_[0]
-        for peakgroup in self.peakgroups_:
-            if peakgroup[1] <= best_score:
-                best_score = peakgroup[1]
-                result = peakgroup
-        index = [i for i,pg in enumerate(self.peakgroups_) if pg[0] == result[0]][0]
-        return MinimalPeakGroup(result[0], result[1], result[2], self.selected_[index], self, result[3], result[4])
-
-    def get_selected_peakgroup(self):
-      # return the selected peakgroup of this peptide, we can only select 1 or
-      # zero groups per chromatogram!
-      selected = [i for i,pg in enumerate(self.selected_) if pg]
-      assert len(selected) < 2
-      if len(selected) == 1:
-        index = selected[0]
-        result = self.peakgroups_[index]
-        return MinimalPeakGroup(result[0], result[1], result[2], self.selected_[index], self, result[3], result[4])
-      else: 
-          return None
-
-    def get_all_peakgroups(self):
-        for index, result in enumerate(self.peakgroups_):
-            yield MinimalPeakGroup(result[0], result[1], result[2], self.selected_[index], self, result[3], result[4])
-  
-    def find_closest_in_iRT(self, delta_assay_rt):
-      result = min(self.peakgroups_, key=lambda x: abs(float(x[2]) - float(delta_assay_rt)))
-      index = [i for i,pg in enumerate(self.peakgroups_) if pg[0] == result[0]][0]
-      return MinimalPeakGroup(result[0], result[1], result[2], self.selected_[index], self, result[3], result[4])
-
-class Run():
-    """
-    One single SWATH run that contains peptides (chromatograms) 
-    It has a unique id and stores the headers from the csv
-
-    A run has the following attributes: 
-        - an identifier that is unique to this run
-        - a filename where it originally came from
-        - a dictionary of precursors, accessible through a dictionary
-    """
-
-    def __init__(self, header, header_dict, runid, orig_input_filename=None, filename=None, aligned_filename=None):
-        self.header = header
-        self.header_dict = header_dict
-        self.runid = runid
-        self.orig_filename = orig_input_filename # the original input filename
-        self.openswath_filename = filename # the original OpenSWATH filename
-        self.aligned_filename = aligned_filename # the aligned filename
-        self.all_peptides = {}
-  
-    def get_id(self):
-        return self.runid
-
-    def get_openswath_filename(self):
-        return self.openswath_filename
-
-    def get_aligned_filename(self):
-        return self.aligned_filename
-  
-    def get_best_peaks(self):
-        result = []
-        for k, peptide in self.all_peptides.iteritems():
-          result.append(peptide.get_best_peakgroup())
-        return result
-  
-    def get_best_peaks_with_cutoff(self, cutoff):
-        return [p for p in self.get_best_peaks() if p.get_fdr_score() < cutoff]
-  
-    def get_all_trgroups(self, cutoff):
-      above_cutoff = []
-      for k,peak in self.all_peptides.iteritems():
-        if peak.get_fdr_score() < cutoff:
-            above_cutoff.append(peak)
-      return above_cutoff
-  
-    def get_peptide(self, id):
-        try:
-          return self.all_peptides[id]
-        except KeyError:
-          # this run has no peakgroup for that peptide
-          return None
-
-    def __iter__(self):
-        for peptide in self.all_peptides.values():
-            yield peptide
+from msproteomicstoolslib.data_structures.PeakGroup import MinimalPeakGroup, GuiPeakGroup, GeneralPeakGroup
+from msproteomicstoolslib.data_structures.Precursor import GeneralPrecursor, Precursor
+from msproteomicstoolslib.data_structures.Run import Run
 
 class ReadFilter(object):
     """
@@ -601,6 +205,7 @@ class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
         left_width_name = "leftWidth"
         right_width_name = "rightWidth"
         charge_name = "Charge"
+        cluster_id = -1
 
         # use the aligned retention time if it is available!
         if "aligned_rt" in run.header_dict: 
@@ -608,6 +213,8 @@ class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
         # if we want to re-do the re-alignment, we just use the "regular" retention time
         if read_exp_RT: 
             diff_from_assay_in_sec_name = "RT"
+        if "align_clusterid" in run.header_dict: 
+            cluster_id = int(this_row[run.header_dict["align_clusterid"]])
 
         trgr_id = this_row[run.header_dict[unique_peakgroup_id_name]]
         unique_peakgroup_id = this_row[run.header_dict[unique_peakgroup_id_name]]
@@ -645,12 +252,11 @@ class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
 
         if self.readmethod == "minimal":
           peakgroup_tuple = (thisid, fdr_score, diff_from_assay_seconds, intensity, d_score)
-          run.all_peptides[trgr_id].add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id)
+          run.all_peptides[trgr_id].add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id, cluster_id)
         elif self.readmethod == "gui":
           leftWidth = this_row[run.header_dict[left_width_name]]
           rightWidth = this_row[run.header_dict[right_width_name]]
           charge = this_row[run.header_dict[charge_name]]
-          run.all_peptides[trgr_id].charge = charge
           peakgroup = self.PeakGroup(fdr_score, intensity, leftWidth, rightWidth, run.all_peptides[trgr_id])
           run.all_peptides[trgr_id].add_peakgroup(peakgroup)
         elif self.readmethod == "complete":
@@ -659,6 +265,7 @@ class OpenSWATH_SWATHScoringReader(SWATHScoringReader):
           peakgroup.set_fdr_score(fdr_score)
           peakgroup.set_feature_id(thisid)
           peakgroup.set_intensity(intensity)
+          peakgroup.setClusterID(cluster_id)
           run.all_peptides[trgr_id].add_peakgroup(peakgroup)
 
 class mProphet_SWATHScoringReader(SWATHScoringReader):
@@ -694,7 +301,7 @@ class mProphet_SWATHScoringReader(SWATHScoringReader):
             diff_from_assay_in_sec_name = "Tr" 
             diff_from_assay_seconds = float(this_row[run.header_dict["Tr"]]) 
         else:
-            diff_from_assay_seconds = float(this_row[run.header_dict["iRT_empirical"]]) - float(this_row[run.header_dict["iRT_prediced"]])
+            diff_from_assay_seconds = float(this_row[run.header_dict["iRT_empirical"]]) - float(this_row[run.header_dict["iRT_predicted"]])
 
         # create some id
         import uuid 
@@ -717,7 +324,6 @@ class mProphet_SWATHScoringReader(SWATHScoringReader):
           p = self.Precursor(trgr_id, run)
           p.protein_name = protein_name
           p.sequence = sequence
-          p.run_id = run_id
           p.set_decoy(decoy)
           run.all_peptides[trgr_id] = p
         if self.readmethod == "minimal":
@@ -739,11 +345,10 @@ class Peakview_SWATHScoringReader(SWATHScoringReader):
         self.aligned_run_id_name = "align_runid"
         self.readmethod = readmethod
         self.readfilter = readfilter
+
+        # Only minimal reading is implemented
         if readmethod == "minimal":
             self.Precursor = Precursor
-        else:
-            self.Precursor = GeneralPrecursor
-            self.PeakGroup = GeneralPeakGroupPeakView
 
     def parse_row(self, run, this_row, read_exp_RT):
         decoy_name = "Decoy"
@@ -797,13 +402,99 @@ class Peakview_SWATHScoringReader(SWATHScoringReader):
           p = self.Precursor(trgr_id, run)
           p.protein_name = protein_name
           p.sequence = sequence
-          p.run_id = run_id
           p.set_decoy(decoy)
           run.all_peptides[trgr_id] = p
           if verb: print "add peptide", trgr_id
+
+        # Only minimal reading is implemented
         if self.readmethod == "minimal":
           if verb: print "append tuple", peakgroup_tuple
           peakgroup_tuple = (thisid, fdr_score, diff_from_assay_seconds,intensity)
           run.all_peptides[trgr_id].add_peakgroup_tpl(peakgroup_tuple, unique_peakgroup_id)
-        else: raise NotImplemented
+        else:
+            raise NotImplemented
+
+def inferMapping(rawdata_files, aligned_pg_files, mapping, precursors_mapping,
+                 sequences_mapping, verbose=False, throwOnMismatch=False):
+        
+    """ Infers a mapping between raw chromatogram files (mzML) and processed feature TSV files
+
+    Usually on feature file can contain multiple aligned runs and maps to
+    multiple chromatogram files (mzML). This function will try to guess the
+    original name of the mzML based on the align_origfilename column in the
+    TSV. Note that both files have some typical endings that are _not_ shared,
+    these are generally removed before comparison.
+
+    Only an excact match is allowed.
+    """
+    import csv, os
+    for file_nr, f in enumerate(aligned_pg_files):
+        header_dict = {}
+        if f.endswith('.gz'):
+            import gzip 
+            filehandler = gzip.open(f,'rb')
+        else:
+            filehandler = open(f)
+        reader = csv.reader(filehandler, delimiter="\t")
+        header = reader.next()
+        for i,n in enumerate(header):
+            header_dict[n] = i
+        if not header_dict.has_key("align_origfilename") or not header_dict.has_key("align_runid"):
+            print header_dict
+            raise Exception("need column header align_origfilename and align_runid")
+
+        for this_row in reader:
+
+            # Get the mapping ... 
+            if header_dict.has_key("FullPeptideName") and \
+              header_dict.has_key("Charge") and \
+              header_dict.has_key("aggr_Fragment_Annotation"):
+                transitions = this_row[ header_dict["aggr_Fragment_Annotation"] ].split(";")
+                peptide_name = this_row[header_dict["FullPeptideName"]]
+                charge_state = this_row[header_dict["Charge"]]
+                key = peptide_name + "/" + charge_state
+                precursors_mapping [ key ] = transitions
+                mapped_precursors = sequences_mapping.get( peptide_name, [] )
+                mapped_precursors.append(key)
+                sequences_mapping[peptide_name] = [ key ]
+
+            # 1. Get the original filename (find a non-NA entry) and the corresponding run id
+            if len(this_row) == 0: 
+                continue
+            if this_row[ header_dict["align_origfilename"] ] == "NA":
+                continue
+            aligned_id = os.path.basename(this_row[ header_dict["align_runid"] ])
+            if aligned_id in mapping:
+                continue 
+
+            aligned_fname = os.path.basename(this_row[ header_dict["align_origfilename"] ])
+
+            # 2. Go through all chromatogram input files and try to find
+            # one that matches the one from align_origfilename
+            for rfile in rawdata_files:
+
+                # 2.1 remove common file endings from the raw data
+                rfile_base = os.path.basename(rfile)
+                for ending in [".mzML", ".chrom"]:
+                    rfile_base = rfile_base.split(ending)[0]
+
+                # 2.2 remove common file endings from the tsv data
+                for ending in [".tsv", ".csv", ".xls", "_with_dscore", "_all_peakgroups"]:
+                    aligned_fname = aligned_fname.split(ending)[0]
+
+                # 2.3 Check if we have a match
+                if aligned_fname == rfile_base:
+                    if verbose: 
+                        print "- Found match:", os.path.basename(rfile), "->", os.path.basename(this_row[ header_dict["align_origfilename"] ])
+                    mapping[aligned_id] = [rfile]
+
+            if not aligned_id in mapping:
+                if verbose or throwOnMismatch:
+                    print "- No match found for :", aligned_fname, "in any of", \
+                            [os.path.basename(rfile) for rfile in rawdata_files]
+                    print "- This is generally a very bad sign and you might have " +\
+                            "to either rename your files to have matching filenames " +\
+                            "or provide an input yaml file describing the matching in detail"
+                if throwOnMismatch:
+                    raise Exception("Mismatch, alignemnt filename could not be matched to input chromatogram")
 
