@@ -59,7 +59,6 @@ class TestFeatureAlignment(unittest.TestCase):
                 except ValueError:
                     self.assertEqual(field1,field2)
 
-
     def test_1_featureAlignment_openswath(self):
         script = os.path.join(os.path.join(self.scriptdir, "alignment"), "feature_alignment.py")
         filename = os.path.join(self.datadir, "feature_alignment_openswath_input_1.csv")
@@ -100,6 +99,7 @@ class TestFeatureAlignment(unittest.TestCase):
         os.remove(tmpfilename_ids)
         os.remove(tmpfilename_matrix)
 
+    @attr('slow')
     def test_3_featureAlignment_openswath_alignment(self):
         script = os.path.join(os.path.join(self.scriptdir, "alignment"), "feature_alignment.py")
         filename = os.path.join(self.datadir, "feature_alignment_3_openswath_input.csv")
@@ -107,7 +107,7 @@ class TestFeatureAlignment(unittest.TestCase):
         tmpfilename = "featureAlignment_3.out.tmp"
         tmpfilename_ids = "featureAlignment_3.out.tmp_idsonly.csv"
 
-        args = "--in %s --out %s --out_ids %s --realign_runs --method best_cluster_score --max_fdr_quality 0.4 --matrix_output_method RT" % (filename, tmpfilename, tmpfilename_ids)
+        args = "--in %s --out %s --out_ids %s --realign_method 'splineR' --method best_cluster_score --max_fdr_quality 0.4 --matrix_output_method RT" % (filename, tmpfilename, tmpfilename_ids)
         cmd = "python %s %s" % (script, args)
         sub.check_output(cmd,shell=True)
         
@@ -124,7 +124,7 @@ class TestFeatureAlignment(unittest.TestCase):
         tmpfilename = "featureAlignment_4.out.tmp"
         tmpfilename_ids = "featureAlignment_4.out.tmp_idsonly.csv"
 
-        args = "--in %s --out %s --out_ids %s --realign_runs --use_scikit --method best_cluster_score --max_fdr_quality 0.4 --matrix_output_method RT" % (filename, tmpfilename, tmpfilename_ids)
+        args = "--in %s --out %s --out_ids %s --realign_method 'splinePy' --method best_cluster_score --max_fdr_quality 0.4 --matrix_output_method RT" % (filename, tmpfilename, tmpfilename_ids)
         cmd = "python %s %s" % (script, args)
         sub.check_output(cmd,shell=True)
         
@@ -140,7 +140,7 @@ class TestFeatureAlignment(unittest.TestCase):
         tmpfilename = "featureAlignment_5.out.tmp"
         tmpfilename_matrix = "featureAlignment_5.out.tmp_matrix.tsv"
 
-        args = "--in %s --out %s --out_matrix %s --file_format peakview  --outlier_thresh 5 --max_fdr_quality 0.0001 --fdr_cutoff 0.000000001 --method best_cluster_score --matrix_output_method RT" % (filename, tmpfilename, tmpfilename_matrix)
+        args = "--in %s --out %s --out_matrix %s --file_format peakview  --max_fdr_quality 0.0001 --fdr_cutoff 0.000000001 --method best_cluster_score --matrix_output_method RT" % (filename, tmpfilename, tmpfilename_matrix)
         cmd = "python %s %s" % (script, args)
         sub.check_output(cmd,shell=True)
         
@@ -148,5 +148,25 @@ class TestFeatureAlignment(unittest.TestCase):
 
         os.remove(tmpfilename_matrix)
 
+    def test_6_featureAlignment_openswath_best_overall(self):
+        script = os.path.join(os.path.join(self.scriptdir, "alignment"), "feature_alignment.py")
+        filename = os.path.join(self.datadir, "feature_alignment_openswath_input_1.csv")
+        expected_outcome = os.path.join(self.datadir, "feature_alignment_6_output_1_ids.csv")
+        expected_matrix_outcome = os.path.join(self.datadir, "feature_alignment_6_output_2_matrix.csv")
+        tmpfilename = "featureAlignment_6.out.tmp"
+        tmpfilename_ids = "featureAlignment_6.out.tmp_idsonly.csv"
+        tmpfilename_matrix = "featureAlignment_6.out.tmp_matrix.tsv"
+
+        args = "--in %s --out %s --out_ids %s --out_matrix %s --method LocalMSTAllCluster --max_fdr_quality 0.4 --matrix_output_method RT" % (filename, tmpfilename, tmpfilename_ids, tmpfilename_matrix)
+        cmd = "python %s %s" % (script, args)
+        sub.check_output(cmd,shell=True)
+        
+        self.exact_diff(tmpfilename_ids, expected_outcome)
+        self.exact_diff(tmpfilename_matrix, expected_matrix_outcome)
+
+        os.remove(tmpfilename)
+        os.remove(tmpfilename_ids)
+        os.remove(tmpfilename_matrix)
+
 if __name__ == '__main__':
     unittest.main()
