@@ -88,7 +88,7 @@ class TestUnitSmoothing(unittest.TestCase):
 
     @attr('slow')
     def test_smooth_spline_scikit_wrap(self):
-        """Test the smoothing spline using scikit"""
+        """Test the smoothing spline using scikit and the wrapper"""
         sm = smoothing.SmoothingPy()
         import numpy
         sm.initialize(self.data1, self.data2, xmin=numpy.min(numpy.array(self.data1)), xmax=numpy.max(numpy.array(self.data1)))
@@ -136,6 +136,72 @@ class TestUnitSmoothing(unittest.TestCase):
         for res, exp in zip(r,expected):
             self.assertAlmostEqual(res,exp)
 
+    def test_smooth_spline_scipy_uni(self):
+        """Test the univariate spline using spline (no crossvalidation)"""
+        sm = smoothing.UnivarSplineNoCV()
+        sm.initialize(self.data1, self.data2)
+        r = sm.predict(self.data1)
+        self.assertEqual(len(r), 8)
+
+        expected =  [4.1457135130652265, 7.442333705513172, 8.9177273726462474, 10.248891199849604, 11.414111021721407, 13.991054262042756, 7.5957445613093642, 5.8444243638522186]
+        self.assertEqual(len(r), 8)
+        for a,b in zip(expected,r):
+          self.assertAlmostEqual(a, b)
+
+        r = sm.predict(self.data2)
+        expected = [2.34266247,   7.2926131 ,  10.48943975,  11.85840597,
+                11.85840597,  13.48225519,   7.44184246,   6.61579704]
+
+        self.assertEqual(len(r), 8)
+        for a,b in zip(expected,r):
+          self.assertTrue( abs(1-a/b) < 0.05)
+
+    def test_smooth_spline_scipy_cv(self):
+        """Test the univariate spline using spline (with crossvalidation)"""
+        sm = smoothing.UnivarSplineCV()
+        sm.initialize(self.data1, self.data2)
+        r = sm.predict(self.data1)
+        self.assertEqual(len(r), 8)
+
+        # This is slightly better than the NoCV variation
+        expected =  [4.1626385843797094, 7.3804099239612029, 8.9667396489152544, 10.384851777100122, 11.316311505414465, 13.994282700490476, 7.5367306411050095, 5.8580352186337672]
+        for a,b in zip(expected,r):
+            self.assertTrue( abs(1.0-a/b) < 0.1)
+
+    def test_smooth_nn(self):
+        """Test the univariate spline using local kernel smoothing"""
+        sm = smoothing.WeightedNearestNeighbour(3, 5, 0.5, False)
+        sm.initialize(self.data1, self.data2)
+        r = sm.predict(self.data1)
+        self.assertEqual(len(r), 8)
+
+        # This is slightly better than the NoCV variation
+        expected = [5.2723735408560302, 7.5434782608695654, 8.6875, 9.5625, 10.75, 15.6, 7.6671586996151504, 6.2922201138519931]
+        for a,b in zip(expected,r):
+            self.assertAlmostEqual(a, b)
+
+        sm = smoothing.WeightedNearestNeighbour(3, 5, 0.1, False)
+        sm.initialize(self.data1, self.data2)
+        r = sm.predict(self.data1)
+        self.assertEqual(len(r), 8)
+
+        # This is slightly better than the NoCV variation
+        expected = [5.4637421665174575, 8.0223880597014929, 9.2750000000000004, 9.78125, 10.750000000000002, 15.6, 8.1320351120742185, 6.4280968201233994]
+        for a,b in zip(expected,r):
+            self.assertAlmostEqual(a, b)
+
+    def test_smooth_lowess(self):
+        """Test the lowess smoothing"""
+        sm = smoothing.LowessSmoothingPy()
+        sm.initialize(self.data1, self.data2)
+        r = sm.predict(self.data1)
+        self.assertEqual(len(r), 8)
+
+        # This is slightly better than the NoCV variation
+        expected = [4.2123769729879061, 7.3305230831706876, 8.8162015867770727, 10.144542883530072, 11.507036080352814, 14.061195393451431, 7.4880128821482463, 5.7476045207327786]
+        for a,b in zip(expected,r):
+            self.assertAlmostEqual(a, b)
+
     def test_smooth_spline_r(self):
         """Test the smoothing spline using R"""
         sm = smoothing.SmoothingR()
@@ -149,7 +215,7 @@ class TestUnitSmoothing(unittest.TestCase):
           self.assertAlmostEqual(a, b)
 
     def test_smooth_spline_r_extern(self):
-        """Test the smoothing spline using R"""
+        """Test the smoothing spline using external R"""
         sm = smoothing.SmoothingRExtern()
         sm.initialize(self.data1, self.data2)
         r = sm.predict(self.data2)
@@ -161,6 +227,9 @@ class TestUnitSmoothing(unittest.TestCase):
           self.assertAlmostEqual(a, b)
 
     def test_duplication(self):
+        """
+        Test de-duplication of array data
+        """
         arr = [0, 0, 5, 6, 6, 7, 8, 8]
         sm = smoothing.SmoothingPy()
         de_dupl,duplications = sm.de_duplicate_array(arr)
@@ -169,6 +238,9 @@ class TestUnitSmoothing(unittest.TestCase):
         self.assertEqual(re_dupl, arr)
 
     def test_gettingOperator(self):
+        """
+        Test getting the correct smoothing operator
+        """
         op = smoothing.get_smooting_operator()
         self.assertTrue(isinstance(op, smoothing.SmoothingR))
 
@@ -181,5 +253,22 @@ class TestUnitSmoothing(unittest.TestCase):
         op = smoothing.get_smooting_operator(use_external_r=True, tmpdir="tmp")
         self.assertTrue(isinstance(op, smoothing.SmoothingRExtern))
 
+    def test_gettingOperator_obj(self):
+        """
+        Test getting the correct smoothing operator (new interface)
+        """
+
+        op = smoothing.getSmoothingObj("diRT")
+        self.assertTrue(isinstance(op, smoothing.SmoothingNull))
+
+        op = smoothing.getSmoothingObj("None")
+        self.assertTrue(isinstance(op, smoothing.SmoothingNull))
+
+        op = smoothing.getSmoothingObj("linear")
+        self.assertTrue(isinstance(op, smoothing.SmoothingLinear))
+
+        op = smoothing.getSmoothingObj("splineR")
+        self.assertTrue(isinstance(op, smoothing.SmoothingR))
+
 if __name__ == '__main__':
     unittest.main()
